package com.slideme.demo.controller;

import android.app.Activity;
import android.app.AlarmManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.SystemClock;
import android.util.Log;
import android.view.View;
import android.widget.*;
import com.slideme.demo.model.C;
import com.slideme.demo.model.interaction.SamInteractor;
import com.slideme.demo.model.slidelock.LicenseCheckAsyncTask;
import com.slideme.demo.R;
import com.slideme.slidelock.License;

import java.io.*;
import java.util.Timer;
import java.util.TimerTask;

public class SlideLockedActivity extends Activity {


    private static AsyncTask licenseCheck = null;
    private static final long TRIAL_ALLOW_MAX = C.Time.Unit.MINUTE * 1;

    public void onCreate(Bundle savedInstanceState) {
        setTitle("SlideLock Demo");
        setContentView(R.layout.slidelocked);
        super.onCreate(savedInstanceState);

        findViewById(R.id.q_submit).setOnClickListener(new View.OnClickListener() {
            public void onClick(View view) {
                final String licenseKey = (((EditText) findViewById(R.id.q_input)).getText().toString());

                if (licenseCheck == null)
                    licenseCheck = new LicenseCheckAsyncTask(SlideLockedActivity.this, licenseKey) {
                        public void onPostExecute(License.Rights rights) {
                            if (rights == null) {
                                Toast.makeText(SlideLockedActivity.this, "LICENSE NOK", Toast.LENGTH_LONG).show();

                                onLicenseNOK(licenseKey);
                            } else {
                                Toast.makeText(SlideLockedActivity.this, "License OK", Toast.LENGTH_LONG).show();
                            }
                            licenseCheck = null;
                        }
                    }.execute();
                else
                    Log.w("LICENSE.CHECK#2", "Already checking for license");
            }
        });

    }

    private void onLicenseNOK(String licenseKey) {
        FileInputStream fis = null;
        try {
            DataInputStream dis = new DataInputStream((fis = openFileInput(licenseKey + "trial.ts")));
            long firstStart = dis.readLong();
            long timeElapsed = System.currentTimeMillis() - firstStart;
            Log.d("LICENSE.TRIAL", "Time elapsed since first run under " + licenseKey + " = " + timeElapsed / C.Time.Unit.SECOND + "s");
            if (timeElapsed >= TRIAL_ALLOW_MAX) {
                Log.e("LICENSE.TRIAL", "Trial period exceeded. We may want to shutdown the app?");
            } else {
                scheduleShutDown(TRIAL_ALLOW_MAX - timeElapsed);
            }
        } catch (FileNotFoundException e) {/* if not found, this is  the first run of the application*/
            markFirstRun(licenseKey);
            scheduleShutDown(TRIAL_ALLOW_MAX);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (fis != null)
                try {
                    fis.close();
                } catch (IOException ignored) {
                }
        }
    }

    private Timer shutdown = null;

    private synchronized void scheduleShutDown(long delay) {
        if (shutdown != null) {
            Log.d("LICENSE.TRIAL", "Cancelling previous shutdown schedule");
            shutdown.cancel();
        }
        shutdown = new Timer();
        shutdown.schedule(new TimerTask() {
            @Override
            public void run() {
                Log.d("LICENSE.TRIAL#SHUTDOWN", "Trial period exceeded");

                /* SAM will show Opera application. You can find your bundleId from application details page and can use your own.*/
                Intent querySlideME = new SamInteractor(getPackageManager()).query("market://search?bundleId=3ee705e6-1101-102c-9253-000f4b0782e8", true);

                PendingIntent restart = PendingIntent.getActivity(SlideLockedActivity.this, (int) System.currentTimeMillis(), querySlideME, Intent.FLAG_ACTIVITY_NEW_TASK);
                AlarmManager alarmManager = (AlarmManager) SlideLockedActivity.this.getSystemService(Context.ALARM_SERVICE);
                alarmManager.set(AlarmManager.ELAPSED_REALTIME_WAKEUP, SystemClock.elapsedRealtime() + C.Time.Unit.SECOND * 3, restart);/* Schedule restart in 3 seconds ?*/
                Log.d("LICENSE.TRIAL#SHUTDOWN", "Scheduled SAM Details page");
                /* wait a second to display toast */
                System.exit(16899);
            }
        }, delay); /* Restart the application after allowed trial time has expired*/
        Log.d("LICENSE.TRIAL#2", "Shutdown reaper scheduled in" + delay / C.Time.Unit.SECOND + "s");
    }

    private void markFirstRun(String licenseKey) {
        long now = System.currentTimeMillis();
        Log.d("TRIAL", "Marking first run timestamp for " + licenseKey + " : " + now);
        FileOutputStream fos = null;
        try {
            DataOutputStream dos = new DataOutputStream((fos = openFileOutput(licenseKey + "trial.ts", MODE_PRIVATE)));
            dos.writeLong(now);
        } catch (FileNotFoundException e1) {
            e1.printStackTrace();
        } catch (IOException e1) {
            e1.printStackTrace();
        } finally {
            if (fos != null)
                try {
                    fos.close();
                } catch (IOException ignored) {
                }
        }
    }
}
