package com.slideme.demo.model.interaction;


import android.content.Intent;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.ResolveInfo;
import android.net.Uri;

import java.util.List;

/**
 * This class provides basic functionality for firing Intents
 * It also provides specific functionality for detecting if SlideME SAM is installed on device
 * And the ability to enforce an intent to be handled by SlideME SAM
 */
public class SamInteractor {

    private static final String SAM_PACKAGE_NAME = "com.slideme.sam.manager";

    private final PackageManager pm;

    public SamInteractor(PackageManager pm) {
        this.pm = pm;
    }

    public PackageInfo getSAMPackage() {
        try {
            return pm.getPackageInfo(SAM_PACKAGE_NAME, PackageManager.GET_ACTIVITIES);
        } catch (PackageManager.NameNotFoundException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * @param uriStr intent URI
     * @return An ACTION_VIEW intent with <code>uriStr</code> URI
     */
    public Intent query(String uriStr) {
        return query(uriStr, false);
    }

    /**
     * @param uriStr  intent URI
     * @param enforce flag for enforcing SAM to handle the intent (if SAM is present on device)
     * @return An ACTION_VIEW intent with <code>uriStr</code> URI
     */
    public Intent query(String uriStr, boolean enforce) {
        Intent anIntent = new Intent(Intent.ACTION_VIEW);
        anIntent.setData(Uri.parse(uriStr));
        if (enforce)
            enforceSAM(anIntent);
        return anIntent;
    }

    /**
     * Overrides the intent to be handled by
     *
     * @param original an intent that is to be handled by SlideME SAM
     * @return The original intent with overridden className, when SAM is available on the device
     */
    public boolean enforceSAM(Intent original) {
        List<ResolveInfo> packs = pm.queryIntentActivities(original, 0);
        if (packs != null) {
            for (ResolveInfo pack : packs) {
                if (SAM_PACKAGE_NAME.equals(pack.activityInfo.packageName)) {
                    original.setClassName(pack.activityInfo.packageName, pack.activityInfo.name);
                    return true;
                }
            }
        }
        return false;
    }
}
